searchData={"items":[{"type":"module","title":"Telemetria","doc":"`Telemetría` is the opinionated wrapper for [`:telemetry`](https://hexdocs.pm/telemetry)\n(started with `v0.19.0` it became agnostic to the actual telemetry backend and supports\n`OpenTelemetry` out of the box, allowing for more custom implementations of the said backend.)\nIt provides handy macros to attach telemetry events to any function, private function,\nanonymous functions (on per-clause basis) and just random set of expressions.\n\n`Telemetría` exports three macros:\n\n- `deft/2` which is wrapping `Kernel.def/2`\n- `defpt/2` which is wrapping `Kernel.defp/2`\n- `t/2` which is wrapping the expression passed as the first parameter\n  and adds the options passed as a keyword to the second parameter to the\n  context of the respective telemetry event\n\n>","ref":"Telemetria.html"},{"type":"module","title":"Module attribute over macros {: .tip} - Telemetria","doc":">\n> Unless inevitably needed, one should prefer module attributes over explicit macros\n> (see the section “Using Module Attribute” below.)\n>\n> Module attributes have a way richer customization abilities, including but \n> not limited to conditional wrapping, slack messaging etc. See options\n> which are accepted by the module attribute below.\n\n`Telemetría` allows compile-time telemetry events definition and provides\na compiler that is responsible for incremental builds and updates of the list of\nevents telemetry is aware about.\n\n>","ref":"Telemetria.html#module-module-attribute-over-macros-tip"},{"type":"module","title":"Compile-time config {: .warning} - Telemetria","doc":">\n> `Telemetría` uses a compiler to wrap annotated functions with a telemetry calls.\n> That means, that all the configuration must be placed into compile-time config files.","ref":"Telemetria.html#module-compile-time-config-warning"},{"type":"module","title":"Using Module Attribute - Telemetria","doc":"Besides the functions listed above, one might attach `Telemetría` to the function\nby annotating it with `@telemetria` module attribute.\n\nThere are several options to pass to this attribute:\n\n- **`true`** — attach the `telemetry` to the function\n- **`if: boolean()`** — compile-time condition\n- **`if: (result -> boolean())`** — runtime condition\n- **`level: Logger,level()`** — specify a min logger level to attach telemetry\n- **`group: atom()`** — the configured group to manage event throttling,\n  see `:throttle` setting in `Telemetria.Options`\n- **`locals: [atom()]`** — the list of names of local variables to be exported\n  to the telemetry call\n- **`transform: [{:args, (list() -> list())}, {:result, (any() -> any())}]`** — \n  the functions to be called on the incoming attributes and/or result to reshape them\n- **`reshape: (map() -> map())`** — the function to be called on the resulting attributes\n  to reshape them before sending to the actual telemetry handler; the default application-wide\n  reshaper might be set in `:telemetria, :reshaper` config\n- **`messenger_channels: %{optional(atom()) => {module, keyword()}`** — more handy messenger\n  management, several channels config with channels names associated with their\n  implementations and properties","ref":"Telemetria.html#module-using-module-attribute"},{"type":"module","title":"Example - Telemetria","doc":"The following code would emit the telemetry event for the function `weather`,\n  returning `result` in Celcius _and_ injecting `farenheit` value under `locals`\n\n```elixir\ndefmodule Forecast do\n  use Telemetria\n\n  @telemetria level: :info, group: :weather_reports, locals: [:farenheit]\n  def weather(city) do\n    fahrenheit = ExternalService.retrieve(city)\n    Converter.fahrenheit_to_celcius(fahrenheit)\n  end\nend\n```","ref":"Telemetria.html#module-example"},{"type":"module","title":"Advantages - Telemetria","doc":"`Telemetría` takes care about managing events in the target application,\nmakes it a single-letter change to turn a function into a function wrapped\nwith telemetry call, measuring the execution time out of the box.\n\nIt also allows to easily convert expressions to be be telemetry-aware.\n\nBesides that, `telemetry: false` flag allows to purge the calls in compile-time\nresulting in zero overhead (useful for benchmark, or like.)","ref":"Telemetria.html#module-advantages"},{"type":"module","title":"Example - Telemetria","doc":"You need to include the compiler in `mix.exs`:\n\n```elixir\ndefmodule MyApp.MixProject do\n  def project do\n    [\n      # ...\n      compilers: [:telemetria | Mix.compilers()],\n      # ...\n    ]\n  end\n  # ...\nend\n```","ref":"Telemetria.html#module-example"},{"type":"module","title":"Enabling `Telemetría` - Telemetria","doc":"To enable `telemetría` for the project, you should add `:telemetria` compiler to the list\nof `Mix.compilers/0` as shown below (`mix.exs`).\n\n```elixir\ndef project do\n  [\n    ...\n    compilers: [:telemetria | Mix.compilers()],\n    ...\n  ]\nend\n```\n\nAdditional steps are described below for the different use-cases.","ref":"Telemetria.html#module-enabling-telemetri-a"},{"type":"module","title":"Plain Macros - Telemetria","doc":"In the modules you want to add telemetry macros to, you should `require Telemetria` (or,\npreferably, `import Telemetria` to make it available without FQN.) Once imported,\nthe macros are available and tracked by the compiler.\n\n```elixir\ndefmodule MyMod do\n  import Telemetria\n\n  defpt pi, do: 3.14\n  deft answer, do: 42 - pi()\n\n  def inner do\n    short_result = t(42 * 42)\n    result =\n      t do\n        # long calculations\n        :ok\n      end\n  end\nend\n```","ref":"Telemetria.html#module-plain-macros"},{"type":"module","title":"Module Attribute - Telemetria","doc":"Module attributes are processed by the compilation hooks. To enable `@telemetria`\nmodule attributes, one should `use Telemetria`. Below is the example that would send\ntwo telemetry events to the configured `Telemetria.Backend`.\n\n```elixir\ndefmodule Otel do\n  @moduledoc \"`Telemetria` with :opentelemetry` example\"\n\n  use Telemetria\n\n  @telemetria level: :info, group: :weather_reports, locals: [:celsius], messenger: :slack\n  def f_to_c(fahrenheit) do\n    celsius = do_f_to_c(fahrenheit)\n    round(celsius)\n  end\n\n  @telemetria level: :info, group: :weather_reports\n  defp do_f_to_c(fahrenheit), do: (fahrenheit - 32) * 5 / 9\nend\n```","ref":"Telemetria.html#module-module-attribute"},{"type":"module","title":"Typical Config - Telemetria","doc":"`Telemetría` requires an application-wide config to operate properly. Yes, I know\nhaving a config in a library is discouraged by the core team. Unfortunately, for the\ncompiler to work properly, the static compile-time config is still required.\n\nAfter all, even if running many OTP applications on the same node, one would barely\nwant to have a different telemetry config for them.\n\n```elixir\nimport Config\n\nconfig :telemetria,\n  purge_level: :debug,\n  level: :info,\n  events: [\n    [:tm, :f_to_c]\n  ],\n  throttle: %{some_group: {1_000, :last}}\n# create a slack app and put URL here\n# messenger_channels: %{slack: {:slack, url: \"\"}}\n```","ref":"Telemetria.html#module-typical-config"},{"type":"module","title":"Use in releases - Telemetria","doc":"`:telemetria` compiler keeps track of the events in the compiler manifest file\nto support incremental builds. Also it spits out `config/.telemetria.config.json`\nconfig for convenience. It might be used in in the release configuration as shown below.\n\n```elixir\nreleases: [\n  configured: [\n    # ...,\n    config_providers: [{Telemetria.ConfigProvider, \"/etc/telemetria.json\"}]\n  ]\n]\n```","ref":"Telemetria.html#module-use-in-releases"},{"type":"module","title":"Options - Telemetria","doc":"* `:otp_app` (`t:atom/0`) - OTP application this telemetry is attached to. The default value is `:telemetria`.\n\n* `:enabled` (`t:boolean/0`) - Specifies whether telemetry should be enabled. The default value is `true`.\n\n* `:backend` (`t:atom/0`) - The backend to be used as an actual implementation The default value is `Telemetria.Backend.Telemetry`.\n\n* `:messenger_channels` (`t:map/0`) - The messenger channels as a map `%{name => {impl, opts}}` The default value is `%{}`.\n\n* `:level` - Telemetria level to skip logging beyond, as in [Logger](https://hexdocs.pm/logger/Logger.html#module-application-configuration) The default value is `:debug`.\n\n* `:purge_level` - Telemetria level to purge beyond, as in [Logger](https://hexdocs.pm/logger/Logger.html#module-application-configuration) The default value is `:debug`.\n\n* `:throttle` - The throttling mechanism for throttling through too many events The default value is `:none`.\n\n* `:strict` (`t:boolean/0`) - Ignore `@telemetria` tags without `if` clause The default value is `false`.\n\n* `:smart_log` (`t:boolean/0`) - Log format to use; when true, custom json would be used The default value is `false`.\n\n* `:applications` (`t:keyword/0`) - List the applications to enable Telemetria support for, with parameters The default value is `[]`.\n\n* `:json_config_path` (`t:String.t/0`) - Relative path to JSON config The default value is `\"config/.telemetria.config.json\"`.\n\n* `:events` - The application-specific events.\n\n  See `t:Telemetria.event_prefix/0` and `t:Telemetria.event_name/0`.\n\n  The default value is `[]`.\n\n* `:handler` - Event handler for this application’s telemetry events. Arity must be 4. The default value is `{Telemetria.Handler.Default, :handle_event}`.\n\n* `:polling` (`t:keyword/0`) - The default value is `[enabled: false, flush: 5000, poll: 5000]`.\n\n  * `:enabled` (`t:boolean/0`) - Specifies whether polling should be enabled. The default value is `true`.\n\n  * `:flush` (`t:non_neg_integer/0`) - Flush interval. The default value is `5000`.\n\n  * `:poll` (`t:non_neg_integer/0`) - Poll interval. The default value is `5000`.\n\n* `:process_info` (`t:boolean/0`) - Specifies whether each telemetry event should include process info. The default value is `false`.","ref":"Telemetria.html#module-options"},{"type":"macro","title":"Telemetria.defpt/2","doc":"Declares a private function with a telemetry attached, measuring execution time","ref":"Telemetria.html#defpt/2"},{"type":"macro","title":"Telemetria.deft/2","doc":"Declares a function with a telemetry attached, measuring execution time","ref":"Telemetria.html#deft/2"},{"type":"macro","title":"Telemetria.t/2","doc":"Attaches telemetry to anonymous function (per clause,) or to expression(s)","ref":"Telemetria.html#t/2"},{"type":"function","title":"Telemetria.telemetry_prefix/2","doc":"","ref":"Telemetria.html#telemetry_prefix/2"},{"type":"type","title":"Telemetria.event_measurements/0","doc":"","ref":"Telemetria.html#t:event_measurements/0"},{"type":"type","title":"Telemetria.event_metadata/0","doc":"","ref":"Telemetria.html#t:event_metadata/0"},{"type":"type","title":"Telemetria.event_name/0","doc":"","ref":"Telemetria.html#t:event_name/0"},{"type":"type","title":"Telemetria.event_prefix/0","doc":"","ref":"Telemetria.html#t:event_prefix/0"},{"type":"type","title":"Telemetria.event_value/0","doc":"","ref":"Telemetria.html#t:event_value/0"},{"type":"type","title":"Telemetria.handler_config/0","doc":"","ref":"Telemetria.html#t:handler_config/0"},{"type":"behaviour","title":"Telemetria.Backend","doc":"This behaviour should be implemented by the backend, used for actual\ntelemetry events processing.\n\nBackend shipped with the library:\n\n- `Telemetria.Backend.Telemetry`\n- `Telemetria.Backend.OpenTelemetry`","ref":"Telemetria.Backend.html"},{"type":"callback","title":"Telemetria.Backend.entry/1","doc":"The implementation will be called when the block gets entered","ref":"Telemetria.Backend.html#c:entry/1"},{"type":"callback","title":"Telemetria.Backend.exit/1","doc":"The implementation will be called when the block context is to be exited","ref":"Telemetria.Backend.html#c:exit/1"},{"type":"callback","title":"Telemetria.Backend.reshape/1","doc":"The implementation will be called to reshape the event before sending it to the actual handler","ref":"Telemetria.Backend.html#c:reshape/1"},{"type":"callback","title":"Telemetria.Backend.return/2","doc":"The implementation will be called when the block gets exited / executed","ref":"Telemetria.Backend.html#c:return/2"},{"type":"callback","title":"Telemetria.Backend.update/2","doc":"The implementation will be called when the block context is to be updated","ref":"Telemetria.Backend.html#c:update/2"},{"type":"type","title":"Telemetria.Backend.block_context/0","doc":"The context of the currently processing block.\n\nFor instance, `opentelemetry:span_ctx()` for `otel` or `nil` for `telemetry`","ref":"Telemetria.Backend.html#t:block_context/0"},{"type":"type","title":"Telemetria.Backend.block_id/0","doc":"The type used for naming blocks / events","ref":"Telemetria.Backend.html#t:block_id/0"},{"type":"type","title":"Telemetria.Backend.block_metadata/0","doc":"The additional attributes of the block, aside from measurements","ref":"Telemetria.Backend.html#t:block_metadata/0"},{"type":"behaviour","title":"Telemetria.Messenger","doc":"The helper allowing quick sending of the telemetry events to the messenger.","ref":"Telemetria.Messenger.html"},{"type":"callback","title":"Telemetria.Messenger.debug/2","doc":"The actual implementation of the message sending (`debug` level)","ref":"Telemetria.Messenger.html#c:debug/2"},{"type":"callback","title":"Telemetria.Messenger.error/2","doc":"The actual implementation of the message sending (`error` level)","ref":"Telemetria.Messenger.html#c:error/2"},{"type":"callback","title":"Telemetria.Messenger.format/2","doc":"The formatter of the incoming message, producing the binary to be sent over the wire","ref":"Telemetria.Messenger.html#c:format/2"},{"type":"callback","title":"Telemetria.Messenger.info/2","doc":"The actual implementation of the message sending (`info` level)","ref":"Telemetria.Messenger.html#c:info/2"},{"type":"function","title":"Telemetria.Messenger.post/3","doc":"Routes the message to the configured messenger(s)","ref":"Telemetria.Messenger.html#post/3"},{"type":"callback","title":"Telemetria.Messenger.warning/2","doc":"The actual implementation of the message sending (`warning` level)","ref":"Telemetria.Messenger.html#c:warning/2"},{"type":"type","title":"Telemetria.Messenger.message/0","doc":"The type of the message the messenger is to process and send","ref":"Telemetria.Messenger.html#t:message/0"},{"type":"module","title":"Telemetria.Backend.Telemetry","doc":"The implementation of `Telemetria.Backend` for `:telemetry`.","ref":"Telemetria.Backend.Telemetry.html"},{"type":"module","title":"Telemetria.Formatter","doc":"JSON formatter that is aware of `Telemetria` metadata.\n\nThis code is gracefully stolen from internal Kantox Observability library.","ref":"Telemetria.Formatter.html"},{"type":"function","title":"Telemetria.Formatter.format/4","doc":"Formats a log entry.","ref":"Telemetria.Formatter.html#format/4"},{"type":"behaviour","title":"Telemetria.Handler","doc":"The behaviour to be implemented by consumers.\n\n`Telemetría` provides the default handler, that simply delegates to `Logger.info/1`.\n\n_See:_ `Telemetria.Handler.Default`.","ref":"Telemetria.Handler.html"},{"type":"callback","title":"Telemetria.Handler.handle_event/4","doc":"The callback that will be invoked by `:telemetry`","ref":"Telemetria.Handler.html#c:handle_event/4"},{"type":"function","title":"Telemetria.Handler.process_info/1","doc":"Collects and formats the current process info to insert to metadata","ref":"Telemetria.Handler.html#process_info/1"},{"type":"type","title":"Telemetria.Handler.process_info/0","doc":"","ref":"Telemetria.Handler.html#t:process_info/0"},{"type":"module","title":"Telemetria.Handler.Default","doc":"Default handler used unless the custom one is specified in config.\n\nThis handler collects `event`, `measurements`, `metadata`, and `config`,\npacks them into the keyword list and logs on `:info` log level.","ref":"Telemetria.Handler.Default.html"},{"type":"module","title":"Telemetria.Telemetry","doc":"The `Supervisor` managing all the internal bundled `telemetry` helpers.","ref":"Telemetria.Telemetry.html"},{"type":"function","title":"Telemetria.Telemetry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Telemetria.Telemetry.html#child_spec/1"},{"type":"function","title":"Telemetria.Telemetry.start_link/1","doc":"Starts this supervisor linked","ref":"Telemetria.Telemetry.html#start_link/1"},{"type":"module","title":"Telemetria.Backend.OpenTelemetry","doc":"The implementation of `Telemetria.Backend` for `OpenTelemetry`.","ref":"Telemetria.Backend.OpenTelemetry.html"},{"type":"exception","title":"Telemetria.Error","doc":"","ref":"Telemetria.Error.html"},{"type":"extras","title":"Telemetría","doc":"# Telemetría    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/telemetria/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/telemetria/workflows/Dialyzer/badge.svg)\n\n**The helper application that simplifies and standardizes telemetry usage.**","ref":"readme.html"},{"type":"extras","title":"Installation - Telemetría","doc":"```elixir\ndef project do\n  [\n    compilers: [:telemetria | Mix.compilers()]\n  ]\nend\n\ndef deps do\n  [\n    {:telemetria, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Changelog - Telemetría","doc":"- **`0.22.0`** —\n  - [FIX] bug fix for `finitomata` integration\n- **`0.21.0`** —\n  - [UPD] support for external messengers (slack)\n- **`0.20.0`** —\n  - [UPD] support for nested spans \n- **`0.19.1`** —\n  - [UPD] `reshape/1` everywhere to make a result transformable\n- **`0.19.0`** —\n  - [UPD] `OpenTelemetry` support through backend abstraction\n- **`0.18.0`** —\n  - [UPD] `locals: [:foo, :bar]` to export local vars from functions\n- **`0.17.0`** —\n  - [UPD] Configurable throttling groups\n- **`0.16.0`** —\n  - [UPD] Elixir v1.17\n- **`0.15.0`** —\n  - [UPD] Elixir v1.16\n- **`0.14.2`** —\n  - [ENH] `process_info: boolean()` option to embed the process info into each telemtry event \n- **`0.14.1`** —\n  - [ENH] Propagate `custom_options: [from: :telemetria]` down to inspect\n- **`0.14.0`** —\n  - [ENH] Allow `if: runtime_function` in `@telemetria` attribute in the form `&Mod.fun/1`,\n    which will receive the result _and_ emit the telemetry event if it returns `true`\n- **`0.13.0`** —\n  - [ENH] Allow `if: compile_time_boolean` in `@telemetria` attribute\n- **`0.12.0`** —\n  - [ENH] Allow `[transform: [args: M.f/1, result: {M, :f}]]` coercers in `@telemetria` attribute\n- **`0.11.1`** —\n  - [ENH] diagnostics improved\n- **`0.11.0`** —\n  - [FIX] properly handle `@telemetria` attribute on function heads and multiple clauses\n  - [ENH] purge level for telemetria hooks\n  - [ENH] level for telemetria logs\n- **`0.9.1`** —\n  - [ENH] `@telemetria process_info: true` keyword parameter\n  - [ENH] `@id` is set to the correct `otp_app`\n  - [ENH] `MFA` is set properly for alerts\n  - [ENH] arguments and result are grouped under `:call`\n  - [ENH] total metadata cleanup\n- **`0.9.0`** —\n  - [ENH] add `Telemetria.Formatter` that can be used to produce JSON logs basing on metadata, natively integrated into `telemetría`\n- **`0.8.0`** —\n  - [ENH] add `applications` option accepting a keyword list of applications to enable `telemetria` for (with optional parameters)\n- **`0.7.0`** —\n  - [ENH] add named arguments in all calls to event’s context\n- **`0.6.1`** —\n  - [BUG] wrong order using telemetry with many clauses\n- **`0.6.0`** —\n  - [ENH] accept parameters in annotation `@telemetria`\n  - [MIN] better print of events added\n- **`0.5.2`** —\n  - [ENH] `enabled: false` config to purge all telemetria code at once\n- **`0.5.1`** —\n  - [ENH] add result of underlying call to metric\n  - [ENH] decrease an amount of garbage returned from caller context\n- **`0.5.0`** —\n  - [ENH] annotation `@telemetria true` as a synonym to `deft/2`\n  - [FIX] polling is off by default\n- **`0.4.0`** —\n  - [ENH] default polling of system / vm states for free\n  - [ENH] starting in phases ensuring proper instrumenter setup\n- **`0.3.0`** —\n  - [ENH] no need for any config in any environment\n  - [BUG] proper handling of guards in compiler, correct event names\n\n## [Documentation](https://hexdocs.pm/telemetria).","ref":"readme.html#changelog"},{"type":"extras","title":"Optional Telemetria Support","doc":"# Optional Telemetria Support\n\nTo optionally include `telemetria` support to your library do the following.\n\n## `mix.exs`\n\nIn your `mix.exs` file, specify the dependency as `optional: true`\n\n```elixir\ndefp deps do\n  ...\n  {:telemetria, \"~> 0.4\", optional: true}\n  ...\nend\n```","ref":"optional-using.html"},{"type":"extras","title":"Custom Module - Optional Telemetria Support","doc":"Declare the custom module `MyApp.Telemetria` as shown below.\n\n```elixir\ndefmodule MyApp.Telemetria do\n  @moduledoc false\n\n  @default_options [use: [], apply: [level: :info]]\n\n  @all_options :telemetria\n               |> Application.get_env(:applications, [])\n               |> Keyword.get(:my_app, [])\n  @options if @all_options == true,\n             do: @default_options,\n             else: Keyword.merge(@default_options, @all_options)\n\n  @use @options != [] and match?({:module, Telemetria}, Code.ensure_compiled(Telemetria))\n\n  defmacro __using__(opts \\\\ []),\n    do: if(@use, do: quote(do: use(Telemetria, unquote(opts))), else: :ok)\n\n  @spec use? :: boolean()\n  def use?, do: @use\n\n  @spec use!(module :: module(), opts :: keyword()) :: :ok | nil\n  def use!(module, opts \\\\ true),\n    do: if(Rambla.Telemetria.use?(), do: Module.put_attribute(module, :telemetria, opts))\nend\n```","ref":"optional-using.html#custom-module"},{"type":"extras","title":"Conditional Include - Optional Telemetria Support","doc":"In the module(s) where you want to optinally use `Telemetria`, add\n`use MyApp.Telemetria`. All the attributes are to be now declared with\n\n```elixir\nif Rambla.Telemetria.use?(), do: @telemetria(level: :info)\n```\n\nor\n\n```elixir\nRambla.Telemetria.use!(__MODULE__, level: :info)\n```\n\nThat’s it. Enjoy.","ref":"optional-using.html#conditional-include"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}